@@ -1,4 +1,4 @@
-/* vi:set ts=8 sts=4 sw=4:
+/* vi:set ts=8 sts=4 sw=4 noet:
  *
  * VIM - Vi IMproved		by Bram Moolenaar
  *				GUI/Motif support by Robert Webb
@@ -9,6 +9,8 @@
  * See README.txt for an overview of the Vim source code.
  */
 
+#include "vim.h"
+
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
 #ifdef FEAT_GUI_NEXTAW
@@ -34,7 +36,6 @@
 # include <X11/Xaw/AsciiText.h>
 #endif /* FEAT_GUI_NEXTAW */
 
-#include "vim.h"
 #ifndef FEAT_GUI_NEXTAW
 # include "gui_at_sb.h"
 #endif
@@ -1960,14 +1961,12 @@ gui_mch_create_scrollbar(
 #endif
 }
 
-#if defined(FEAT_WINDOWS) || defined(PROTO)
     void
 gui_mch_destroy_scrollbar(scrollbar_T *sb)
 {
     if (sb->id != (Widget)0)
 	XtDestroyWidget(sb->id);
 }
-#endif
 
     void
 gui_mch_set_scrollbar_colors(scrollbar_T *sb)
