@@ -51,9 +51,6 @@
 # ifdef _
 #  undef _
 # endif
-# ifdef ngettext
-#  undef ngettext
-# endif
 # ifdef N_
 #  undef N_
 # endif
@@ -655,9 +652,11 @@ gui_mch_add_menu(vimmenu_T *menu, int idx)
     parent_widget = (parent != NULL) ? parent->submenu_id : gui.menubar;
     menu_item_new(menu, parent_widget);
 
+# if !GTK_CHECK_VERSION(3,4,0)
     /* since the tearoff should always appear first, increment idx */
     if (parent != NULL && !menu_is_popup(parent->name))
 	++idx;
+# endif
 
     gtk_menu_shell_insert(GTK_MENU_SHELL(parent_widget), menu->id, idx);
 
@@ -776,10 +775,12 @@ gui_mch_add_menu_item(vimmenu_T *menu, int idx)
 	if (parent == NULL || parent->submenu_id == NULL)
 	    return;
 
+# if !GTK_CHECK_VERSION(3,4,0)
 	/* Make place for the possible tearoff handle item.  Not in the popup
 	 * menu, it doesn't have a tearoff item. */
 	if (!menu_is_popup(parent->name))
 	    ++idx;
+# endif
 
 	if (menu_is_separator(menu->name))
 	{
@@ -1159,7 +1160,6 @@ gui_mch_create_scrollbar(scrollbar_T *sb, int orient)
     }
 }
 
-#if defined(FEAT_WINDOWS) || defined(PROTO)
     void
 gui_mch_destroy_scrollbar(scrollbar_T *sb)
 {
@@ -1170,7 +1170,6 @@ gui_mch_destroy_scrollbar(scrollbar_T *sb)
     }
     gui_mch_update();
 }
-#endif
 
 #if defined(FEAT_BROWSE) || defined(PROTO)
 /*
@@ -1522,7 +1521,7 @@ split_button_string(char_u *button_string, int *n_buttons)
 	    else if (*p == DLG_HOTKEY_CHAR)
 		*p++ = '_';
 	    else
-		mb_ptr_adv(p);
+		MB_PTR_ADV(p);
 	}
 	array[count] = NULL; /* currently not relied upon, but doesn't hurt */
     }
@@ -1955,7 +1954,7 @@ popup_menu_position_func(GtkMenu *menu UNUSED,
 # endif
     {
 	/* Find the cursor position in the current window */
-	*x += FILL_X(W_WINCOL(curwin) + curwin->w_wcol + 1) + 1;
+	*x += FILL_X(curwin->w_wincol + curwin->w_wcol + 1) + 1;
 	*y += FILL_Y(W_WINROW(curwin) + curwin->w_wrow + 1) + 1;
     }
 }
